==> bytekit-core-0.0.7-sources.jar: com/alibaba/bytekit/asm/matcher/SimpleSubclassMatcher.java <==
package com.alibaba.bytekit.asm.matcher;

import java.security.ProtectionDomain;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import com.alibaba.bytekit.utils.ClassLoaderUtils;
import com.alibaba.deps.org.objectweb.asm.ClassReader;

/**
 * 
 * @author hengyunabc 2020-11-24
 *
 */
public class SimpleSubclassMatcher implements ClassMatcher {

    Set<String> classNames = new HashSet<String>();

    public SimpleSubclassMatcher(String... className) {
        for (String name : className) {
            this.classNames.add(name);
        }
    }

    public SimpleSubclassMatcher(Collection<String> names) {
        this.classNames.addAll(names);
    }

    @Override
    public boolean match(ClassLoader loader, String className, Class<?> classBeingRedefined,
            ProtectionDomain protectionDomain, byte[] classfileBuffer) {
        loader = ClassLoaderUtils.wrap(loader);

        if (classBeingRedefined != null) { // 在retransform 时，可以直接判断
            return match(classBeingRedefined);
        } else {
            // 读取出具体的 类名，还有父类名， 如果有匹配，则返回 true，没有，就返回 false
            ClassReader reader = new ClassReader(classfileBuffer);
            String clazzName = reader.getClassName();
            String superName = reader.getSuperName();

            if (classNames != null && classNames.contains(clazzName.replace('/', '.'))) {
                return true;
            }

            while (!superName.equals("java/lang/Object")) {
                if (classNames.contains(superName)) {
                    return true;
                }
                try {
                    Class<?> superClass = loader.loadClass(superName.replace('/', '.'));
                    return match(superClass);
                } catch (ClassNotFoundException e) {
                    // ignore
                    return false;
                }
            }
        }

        return false;
    }

    private boolean match(Class<?> clazz) {
        for (Class<?> superclass = clazz;;) {
            if (superclass == null || Object.class.equals(superclass)) {
                return false;
            }
            if (classNames != null && classNames.contains(superclass.getName().replace('/', '.'))) {
                return true;
            }
            superclass = superclass.getSuperclass();
        }
    }

}

==> bytekit-core-0.1.3-sources.jar: com/alibaba/bytekit/asm/matcher/SimpleSubclassMatcher.java <==
package com.alibaba.bytekit.asm.matcher;

import java.security.ProtectionDomain;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.alibaba.bytekit.asm.meta.ClassMetaService;
import com.alibaba.bytekit.utils.AsmUtils;
import com.alibaba.bytekit.utils.ClassLoaderUtils;

/**
 * 
 * @author hengyunabc 2020-11-24
 *
 */
public class SimpleSubclassMatcher implements ClassMatcher {

    private Set<String> classNames = new HashSet<String>();

    /**
     * 保存另一份转换为 internal 的数据，避免每次match转换
     */
    private Set<String> internalClassNames = new HashSet<String>();

    public SimpleSubclassMatcher(String... className) {
        for (String name : className) {
            add(name);
        }
    }

    public SimpleSubclassMatcher(Collection<String> names) {
        for (String name : names) {
            add(name);
        }
    }

    private void add(String name) {
        classNames.add(name);
        internalClassNames.add(AsmUtils.internalClassName(name));
    }

    @Override
    public boolean match(ClassLoader loader, String className, Class<?> classBeingRedefined,
            ProtectionDomain protectionDomain, byte[] classfileBuffer) {
        loader = ClassLoaderUtils.wrap(loader);

        if (classBeingRedefined != null) { // 在retransform 时，可以直接判断
            return match(classBeingRedefined);
        } else {

            List<String> allSuperNames = ClassMetaService.allSuperNames(loader, className, classfileBuffer);
            for (String superName : allSuperNames) {
                if (internalClassNames.contains(superName)) {
                    return true;
                }
            }
        }

        return false;
    }

    private boolean match(Class<?> clazz) {
        for (Class<?> superclass = clazz;;) {
            if (superclass == null || Object.class.equals(superclass)) {
                return false;
            }
            if (classNames != null && classNames.contains(superclass.getName().replace('/', '.'))) {
                return true;
            }
            superclass = superclass.getSuperclass();
        }
    }

}
